<?php


/*
Plugin Name: SETI@home Stats
Plugin URI: http://www.trassare.com/setihome-stats/
Description: This plugin displays a user's current SETI@home stats anywhere on Wordpress.  A widget is included to easily display stats in a sidebar.
Version: 1.1.9
Author: Samuel T. Trassare
Author URI: http://www.trassare.com
Disclaimer: Use at your own risk. No warranty expressed or implied is provided.
Text Domain: setihome-stats
Domain Path: /languages

*/

/*  Copyright 2017 Samuel T. Trassare (http://www.trassare.com)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

define('SETI_DEBUG', false);
define('SETI_FILE', ABSPATH . 'wp-content/plugins/setihome-stats/seti_cache.txt');

function seti_init() {
	load_plugin_textdomain('setihome-stats', 'wp-content/plugins/setihome-stats');
	define('SETI_ACCT', get_option('seti_acct'));
	define('SETI_EXPY', get_option('seti_expy'));
}

add_action('init', 'seti_init');

function get_seti_stats() {
	// Get the write time
	$fh = fopen(SETI_FILE, 'r');
	$expiry = fread($fh, 10);
	fclose($fh);

	// Date compare
	$today = mktime(date("H"), 0, 0, date("m"), date("d"), date("y"));

	// During debug I like to read the SETI site with each test cycle.
	if (SETI_DEBUG || ($expiry < $today)) {
		//Get Fresh Data
		read_seti_site();
	}

	// Read data from cache
	$fg = fopen(SETI_FILE, 'r');
	$out = fread($fg, filesize(SETI_FILE));
	fclose($fg);

	// Output
	$out = substr($out, 10, strlen($out));
	echo $out;
}

function read_seti_site() {
	$host = 'setiathome.berkeley.edu';
	$path = '/show_user.php?userid=' . SETI_ACCT;
	$seti_logo = get_settings('home') . '/wp-content/plugins/setihome-stats/seti_button.png';
	$seti_url = 'http://setiathome.berkeley.edu';

	$res = "GET $path HTTP/1.0\r\n";
	$res .= "Host: $host\r\n";
	$res .= "User-Agent: " . $_SERVER['HTTP_USER_AGENT'] . "\r\n";
	$res .= "Accept-Language: en-US\r\n";
	$res .= "Content-Type: application/x-www-form-urlencoded\r\n";
	$res .= "Content-Length: 0\r\n";
	$res .= "Connection: close\r\n\r\n";

	$fp = @ fsockopen($host, 80, $errno, $errstr);

	if (!$fp) {
		echo "<ul>\n" . "  <li> Error: $errno </li>\n" . "  <li> Descr: $errstr </li>\n" . "</ul>\n";
		return;
	} else {
		// Write the above HTML header to the socket.
		fputs($fp, $res, strlen($res));
		// Now get the response.
		while (!feof($fp)) {
			$line .= fgets($fp);
		} //while
	} //if

	fclose($fp);

	// $line now contains the entire contents of the file.  Parse it up and write it to cache.

	// If the SETI site is down for maintenance, don't do the update.
	// The following line is for a DEBUG TEST:
	// $line .= 'maintenance';
	if (stristr($line, 'maintenance') === FALSE) {
		// Member name.
		$regexp = '<h2>(.*)<\/h2>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberName = $matches[1];
		}

		// Member's badge, if any.
		$regexp = 'title="Top (.*) in average credit"';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberBadge = $matches[1];
		}

		// Member since.
		$regexp = 'member since<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberSince = $matches[1];
		}

		// Member Country.
		$regexp = 'Country<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberCountry = $matches[1];
		}

		// Member URL.
		$regexp = 'URL<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberUrl = $matches[1];
		}

		// Member total credit.
		$regexp = 'Total credit<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberTotalCredit = $matches[1];
		}

		// Member recent average credit.
		$regexp = 'Recent average credit<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberRecentAverageCredit = $matches[1];
		}

		// Member classic workunits.
		$regexp = 'home classic workunits<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberClassicWorkunits = $matches[1];
		}

		// Member classic cpu time.
		$regexp = 'home classic CPU time<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberClassicCpuTime = $matches[1];
		}

		// Member team.
		$regexp = 'Team<\/td>[ \r\n]+<td style="padding-left:12px" >(.*)<\/td>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$memberTeam = $matches[1];
		}

		$regexp = 'Generated (.*)<\/small>';
		if (preg_match("/$regexp/siU", $line, $matches)) {
			$genTime = $matches[1];
		}

		$expire = mktime(date("H") + SETI_EXPY, 0, 0, date("m"), date("d"), date("y"));

		// Write to cache.
		$fh = fopen(SETI_FILE, 'w') or die("can't open file");
		$stringData = $expire;

		$stringData .= "<p><strong>$memberName</strong></p>\n<ul>\n";

		if ($memberBadge) {
			$stringData .= "  <li>" . __('Top', 'setihome-stats') . " $memberBadge " . __('in average credit', 'setihome-stats') . "</li>\n";
		}
		
		$stringData .= "<li>" . __('Member since', 'setihome-stats') . " $memberSince</li>\n" . "  <li>" . __('Country:', 'setihome-stats') . " $memberCountry</li>\n";

		if ($memberUrl) {
			$stringData .= "  <li><acronym title=\"Uniform Resource Locator\">URL</acronym>: $memberUrl</li>\n";
		}

		$stringData .= "  <li>" . __('Total credit:', 'setihome-stats') . " $memberTotalCredit</li>\n" . "   <li><acronym title=\"Recent Average Credit\">RAC</acronym>: $memberRecentAverageCredit</li>\n" . "   <li>" . __('Classic workunits:', 'setihome-stats') . " $memberClassicWorkunits</li>\n" . "   <li>" . __('Classic', 'setihome-stats') . " <acronym title=\"Central Processing Unit\">CPU</acronym> " . __('time:', 'setihome-stats') . " $memberClassicCpuTime</li>\n";

		if ($memberTeam != "None") {
			$stringData .= "  <li>" . __('Team:', 'setihome-stats') . " $memberTeam</li>\n";
		}

		$stringData .= "<li>S@h Status: <p style=\"display: inline; color: green;\">online</p></li>\n";
		$stringData .= "<li>As of $genTime</li>\n";
		$stringData .= "</ul>\n";
		$stringData .= "<div style=\"border-top: 1px black solid; text-align: center; padding-top: 2px; vertical-align: top;\">
		<a href=\"$seti_url\" title=\"SETI@home\"><img style=\"border-style: none;\" src=\"$seti_logo\" alt=\"Visit SETI@home\" /></a></div>\n";

		fwrite($fh, $stringData);
		fclose($fh);
	} else {
		// Edit the cache to reflect that the SETI@home site is down.
		edit_cache();
	}
}

/**
 * This function will not edit the cache timestamp.  
 * This means that the SETI@home website will be continously 
 * queried until it comes back online.
 */
function edit_cache() {
	$fg = fopen(SETI_FILE, 'r');
	$buf = fread($fg, filesize(SETI_FILE));
	fclose($fg);

	$buf = str_replace("<li>S@h Status: <p style=\"display: inline; color: green;\">online</p></li>", "<li>S@h Status: <p style=\"display: inline; color: red;\">offline</p></li>", $buf);

	$fh = fopen(SETI_FILE, 'w') or die("can't open file");
	fwrite($fh, $buf);
	fclose($fh);
}

/** filters**/
add_action('admin_menu', 'seti_add_options_page');

function seti_add_options_page() {
	add_options_page('SETI@home Stats', 'SETI@home Stats', 'manage_options', 'setihome-stats/options-seti.php');

	$expire = mktime(date("H") - SETI_EXPY, 0, 0, date("m"), date("d"), date("y"));
}
?>
